#include "Arduino.h"

#include "TWI.h"

#include "MS5803_30BA.h"

// Переменная структуры для хранения данных MS5803_30BA. 
MS5803_30BA Ms5803_30Ba;

// Переменные для опроса датчика MS5803-30BA 1 раз в сек.
uint32_t interval = 1000;
uint32_t previousMillis = 0; 

// Переменные для определения максимального времени цикла.
uint32_t timeCycleBegin = 0;
uint32_t timeCycle = 0;
uint32_t timeCycleMax = 0;

/// <summary>
/// Предварительная настройка.
/// </summary>
void setup()
{
  Serial.begin(115200);

  // Инициализация I2C.
  TwiMasterInit();

  // Устанавливаем размер регистра адреса в байтах.
  TwiSetToLengthAddressReg(0);

  // Иницилизация MS5803-30BA.
  MS5803_30BA_Initialize(1);
}

/// <summary>
/// Основной цикл.
/// </summary>
void loop()
{
  // Начальное время цикла.
  //timeCycleBegin = micros();

  // Проверяем часы, каждую секунду.
  uint32_t currentMillis = millis();
  if(currentMillis - previousMillis > interval) 
  {
    previousMillis = currentMillis;

    MS5803_30BA_Begin(&Ms5803_30Ba);
  }

  MS5803_30BA_GetData(&Ms5803_30Ba);

  if (Ms5803_30Ba.valueGet == 3)
  {
    // Show pressure
    Serial.print("Pressure = ");
    Serial.print(Ms5803_30Ba.pressureValue);
    Serial.println(" mbar");

    // Show temperature
    Serial.print("Temperature = ");
    Serial.print(Ms5803_30Ba.temperatureValue);
    Serial.println("C");

    Ms5803_30Ba.valueGet = 4;
  }

  /*
  // Определяем скорость работы.
  timeCycle = GetDifferenceULong(timeCycleBegin, micros());
  // Если текущее значение больше, последнего максимального, отображаем его. 
  if (timeCycle > timeCycleMax)
  {
    timeCycleMax = timeCycle;

    Serial.print("Max - ");
    Serial.println(timeCycleMax);
  }
  */
}

/// <summary>
/// Получение разницы между 2 uint32_t.
/// </summary>
uint32_t GetDifferenceULong(uint32_t BeginTime, uint32_t EndTime)
{
  if (EndTime < BeginTime)
  {
    // Защита от переполнения
    return  - BeginTime + EndTime + 1UL;
  }
  else
  {
    return EndTime - BeginTime;
  }
}

